﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Collections.ObjectModel;
using System.Collections.Specialized;
using DlhSoft.Windows.Data;

namespace GanttChartTaskListViewSimpleDatabaseSample
{
    /// <summary>
    /// Interaction logic for Window1.xaml
    /// </summary>
    public partial class Window1 : Window
    {
        // Data context used to load, store, manage and save the local cached data from/to the database.
        private DataClassesDataContext dataContext;

        // Value used to store the project start date and time computed when loading tasks from the database.
        private DateTime projectStart = DateTime.Today;

        // Value used to store the last task index computed when loading tasks from the database and increased when new tasks are created, in order to maintain the correct order of the tasks in the database.
        private int lastTaskIndex = 0;

        public Window1()
        {
            InitializeComponent();

            // Initialize data.
            LoadData();
        }

        private void LoadMenuItem_Click(object sender, RoutedEventArgs e)
        {
            // Clear and reinitialize data.
            ClearData();
            LoadData();
        }

        private void ClearData()
        {
            // Stop handling new item events.
            GanttChartTaskListView.TaskManager.NewItem -= TaskManager_NewItem;

            // Stop handling collection change events.
            ObservableCollection<Task> tasks = GanttChartTaskListView.ItemsSource as ObservableCollection<Task>;
            tasks.CollectionChanged -= Tasks_CollectionChanged;

            // Clear current data source.
            GanttChartTaskListView.ItemsSource = null;

            // Clear internal fields.
            lastTaskIndex = 0;
            projectStart = DateTime.Today;
            dataContext.Dispose();
        }

        private void LoadData()
        {
            // Initialize data context.
            dataContext = new DataClassesDataContext();

            // Load tasks from the database.
            ObservableCollection<Task> tasks = new ObservableCollection<Task>(
                from t in dataContext.Tasks
                orderby t.Index
                select t);

            // The Task class defines the data object based on the Tasks database table (see DataClasses.dbml).
            // The Task class also defines a set of supplemental properties needed in data binding processing (see DataClasses.cs).
            GanttChartTaskListView.ItemsSource = tasks;

            // Compute and store the project start date and time.
            projectStart = (from t in tasks
                            orderby t.Start
                            select t.Start).FirstOrDefault();

            // Compute and store the last task index.
            lastTaskIndex = (from t in tasks
                             orderby t.Index descending
                             select t.Index).FirstOrDefault();

            // Initialize handling collection change events.
            tasks.CollectionChanged += Tasks_CollectionChanged;

            // Initialize handling new item events.
            GanttChartTaskListView.TaskManager.NewItem += TaskManager_NewItem;

            // Scroll the view to properly display the project.
            DateTime overallStart = GanttChartTaskListView.OverallStart;
            DateTime overallFinish = GanttChartTaskListView.OverallFinish;
            DateTime now = DateTime.Now;
            GanttChartTaskListView.SetTimelinePage(now.AddDays(-1) <= overallStart.AddDays(-1).Add(GanttChartTaskListView.TimelinePageDuration) ? overallStart.AddDays(-1) : (now.AddDays(-1) > overallFinish.AddDays(+1).Add(-GanttChartTaskListView.TimelinePageDuration) ? overallFinish.AddDays(+1).Add(-GanttChartTaskListView.TimelinePageDuration) : now.AddDays(-1)), GanttChartTaskListView.TimelinePageDuration);
            GanttChartTaskListView.ScrollTo(now.AddDays(-1) <= overallStart.AddDays(-1).Add(GanttChartTaskListView.TimelinePageDuration) ? overallStart.AddDays(-1) : (now.AddDays(-1) > overallFinish.AddDays(+1).Add(-GanttChartTaskListView.TimelinePageDuration) ? overallFinish.AddDays(+1) : now.AddDays(-1)));
        }

        private void TaskManager_NewItem(object sender, NewItemEventArgs e)
        {
            // Creates a new task when requested.
            Task task = new Task { Index = ++lastTaskIndex, Content = "New Task", Start = projectStart >= DateTime.Today ? projectStart : DateTime.Today, Effort = TimeSpan.Parse("8:00:00") };
            // Set the new task as the new item to be used.
            e.Item = task;
        }

        private void Tasks_CollectionChanged(object sender, System.Collections.Specialized.NotifyCollectionChangedEventArgs e)
        {
            // Marks adding or deleting of a task to be able to save changes later.
            switch (e.Action)
            {
                case NotifyCollectionChangedAction.Add:
                    foreach (Task t in e.NewItems)
                        dataContext.Tasks.InsertOnSubmit(t);
                    break;
                case NotifyCollectionChangedAction.Remove:
                    foreach (Task t in e.OldItems)
                        dataContext.Tasks.DeleteOnSubmit(t);
                    break;
            }
        }

        private void SaveMenuItem_Click(object sender, RoutedEventArgs e)
        {
            // Save the data changes to the database.
            dataContext.SubmitChanges();
        }
    }
}
